/* -*- C++ -*-
*
* ---------------------------------------------------------------------
* $Id: range.h 43 2016-05-16 22:21:21Z drory $
* ---------------------------------------------------------------------
*
* Copyright (C) 2000-2002 Niv Drory <drory@usm.uni-muenchen.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
*
* ---------------------------------------------------------------------
*
*/




#ifndef __LTL_RANGE__
#define __LTL_RANGE__

#include <ltl/config.h>

#include <climits>   // INT_MIN

LTL_BEGIN_NAMESPACE

enum { minStart = INT_MIN,  minEnd = INT_MIN };

class Range
{
   public:

      Range() :
            first_(minStart),
            last_(minEnd),
            stride_(1),
            length_(1)
      {}

      Range( const int first, const int last, const int stride=1 ) :
            first_(first), last_(last),
            stride_(stride),
            length_((last-first)/stride+1)
      {
         LTL_ASSERT( (first==minStart)||(last>=first && length_>0),
                     "Bad Range("<<first<<","<<last<<")" );
      }

      Range( const Range& other ) :
            first_(other.first_),
            last_(other.last_),
            stride_(other.stride_),
            length_(other.length_)
      { }

      inline int first( int lowest=minStart ) const
      {
         if( first_ == minStart )
            return lowest;
         return first_;
      }

      inline int last( int highest=minEnd ) const
      {
         if( last_ == minEnd )
            return highest;
         return last_;
      }

      inline int length() const
      {
         return length_;
      }

      inline int stride() const
      {
         return stride_;
      }

      static Range all()
      {
         return Range( minStart, minEnd, 1 );
      }

      Range operator-( const int shift ) const
      {
         return Range( first_ - shift, last_ - shift, stride_ );
      }

      Range operator+( const int shift ) const
      {
         return Range(first_ + shift, last_ + shift, stride_);
      }


   private:
      const int first_;
      const int last_;
      const int stride_;
      const int length_;
};

LTL_END_NAMESPACE

#endif // __LTL_RANGE__

