/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: option_parser.h 23 2003-10-28 02:24:16Z drory $
 * ---------------------------------------------------------------------
 *
 * Copyright (C) 2000-2001 Niv Drory <drory@usm.uni-muenchen.de>
 *                         Claus A. Goessl <cag@usm.uni-muenchen.de>
 *                         Arno Riffeser <arri@usm.uni-muenchen.de>
 *                         Jan Snigula <snigula@usm.uni-muenchen.de>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 * ---------------------------------------------------------------------
 *
 */


#ifndef __UTIL_OPTION_PARSER__
#define __UTIL_OPTION_PARSER__

#include <ltl/config.h>

#include <map>
#include <string>
#include <iostream>
#include <fstream>

#include <cstring>

#include <ltl/util/u_exception.h>
#include <ltl/util/option.h>

#ifdef UTIL_USING_NAMESPACE
UTIL_BEGIN_NAMESPACE
using std::string;
using std::ostream;
using std::ofstream;
using std::map;
using std::endl;
#endif


class OptionReader;

// Generic parser object for options. Uses an object of type OptionReader
// to retrieve the options from whereever (config file, command-line, etc.)
// OptionReader is an *abstract* base class defining the following interface:
// OptionReader has to supply exactly three methods, namely
//    string nextOptionName(),
//    string nextOptionValue( const Option* op ),
//    bool   done()
// These are then converted into objects of type Option.
//

class OptionParser
{
   protected:
      friend ostream& operator<<( ostream& os, const OptionParser& op );
      typedef map<string,Option*> omap;
      OptionReader *reader_;

      omap options_;      // holds all options we know of
      omap cmd_options_;  // index of options by command line
      // character
   public:
      OptionParser( OptionReader* reader );
      ~OptionParser();

      OptionParser( const OptionParser& other );

      void changeReader( OptionReader* reader );

      void addOption( Option* option );
      Option* getOption( const string& name );
      void parseOptions() throw( UException );
      void printUsage( ostream& os ) const;
      void writeConfig( const string& filename, const bool withComment = true ) const throw( UException );
      string toString( const bool withComment = false ) const throw( UException );
};


ostream& operator<<( ostream& os, const OptionParser& op );

class OptionReader
{
   public:
      OptionReader()
      { }
      ;

      virtual ~OptionReader()
      { }
      ;

      virtual string nextOptionName()  = 0;
      virtual string nextOptionValue( const Option* op ) = 0;
      virtual bool   done() = 0;
};

#ifdef UTIL_USING_NAMESPACE
UTIL_END_NAMESPACE
#endif

#endif
