/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: region.h 42 2014-09-24 22:58:44Z drory $
 * ---------------------------------------------------------------------
 *
 * Copyright (C) 2000-2001 Niv Drory <drory@usm.uni-muenchen.de>
 *                         Claus A. Goessl <cag@usm.uni-muenchen.de>
 *                         Arno Riffeser <arri@usm.uni-muenchen.de>
 *                         Jan Snigula <snigula@usm.uni-muenchen.de>
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 * ---------------------------------------------------------------------
 *
 */


#ifndef __REGION_H__
#define __REGION_H__

#include <string>

#include <cstdlib>
#include <cstdio>

#include <ltl/config.h>
#include <ltl/util/u_exception.h>

#ifdef UTIL_USING_NAMESPACE
UTIL_BEGIN_NAMESPACE
using std::size_t;
#endif

// region class, holds start and end arrays
// definition
class Region
{
   private:
      Region() 
         : N_(0), start_(NULL), end_(NULL)
      { };
   public:
      Region(const size_t N);
      Region(const Region& other);
      ~Region()
      { delete [] start_; };

      size_t getDim() const
      { return N_; };

      int getStart(const size_t i) const throw(UException);
      int getEnd(const size_t i) const throw(UException);
      size_t getLength() const throw(UException);
      size_t getLength(const size_t i) const throw(UException);
      Region getSlice(const size_t startdim,
                      const size_t enddim) const
         throw(UException);
      void setStart(const size_t i, const int x) throw(UException);
      void setEnd(const size_t i, const int x) throw(UException);
      void setRange(const size_t i, const int s, const int e)
         throw(UException);
      void setRegion(const Region &region) throw(UException);
      string toString() const;

   protected:
      const size_t N_;
      int *const start_;
      int *const end_;

      static const string error_get_dim;
      static const string error_set_dim;
};

#ifdef UTIL_USING_NAMESPACE
UTIL_END_NAMESPACE
#endif

#endif
