/* 
 * background.cpp
 * main routine for background and rms estimation
 *
 * --------------------------------------------------------------------
 * $Id: background.cpp 2 2002-08-19 17:00:29Z drory $
 * --------------------------------------------------------------------
 *
 * Copyright (C) 2000-2001 Niv Drory <drory@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 */

#include "config.h"

#include <string>

#include "globals.h"
#include "yoda.h"

#include <ltl/marray.h>
#include <ltl/fitsio.h>

using namespace ltl;

typedef MArray<float,2> MImage;
typedef MArray<char,2> MMask;

#include "interp.h"


extern Globals g;


void background( MImage& I, MImage& rms, MImage& bg )
{
   // build first iteration background and rms map
   backgroundMap( I, rms, bg );
  
   // if we have to iterate:
   if( g.bg_iter_ )
   {
      cerr << "  Iterating background estimation ...\n";
      // go through thresholding    
      bg += 3*rms; // nice trick ...
      MImage::IndexSet il =
	 where( I >= interp2d( indexPos(I,1), indexPos(I,2), 
			       bg, g.bggrid_, g.bggrid_ ) );
    
      MArray<float,1> save = I(il);

      // set all pixels above 3 sigma to NaN.
      I[il] = g.nanvalue_;
      // and recompute background map
      backgroundMap( I, rms, bg, true );
      I[il] = save;
   }

   // dump background and rms images to disk if requested
   if( g.bg_dump_images_ )
   {
      FitsOut bgfits( prepend_filename( g.infile_, "bg_") );
      FitsOut rmsfits( prepend_filename( g.infile_, "rms_") );

      bgfits.addValueCard( "OBJECT  ", "Background image","Created by YODA" );
      bgfits.addValueCard( "ORIG_IMG", g.infile_, "Original image file" );
      bgfits.setOrigin( "YODA" );
      
      rmsfits.addValueCard( "OBJECT  ", "RMS image (YODA)","Created by YODA" );
      rmsfits.addValueCard( "ORIG_IMG", g.infile_, "Original image file" );
      rmsfits.setOrigin( "YODA" );

      MImage Bg( interp2d( indexPos(I,1), indexPos(I,2), 
			   bg, g.bggrid_, g.bggrid_ ) );
      bgfits << Bg;
      
      Bg = interp2d( indexPos(I,1), indexPos(I,2), 
		     rms, g.bggrid_, g.bggrid_ );
      rmsfits << Bg;
      Bg.free();
   }  
}



