/* -*- C++ -*-
 *
 * photdata.h
 * Declaration of class PhotData
 *
 * --------------------------------------------------------------------
 * $Id: photdata.h 43 2016-05-16 22:21:21Z drory $
 * --------------------------------------------------------------------
 *
 * Copyright (C) 2000-2001 Niv Drory <drory@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 */

#ifndef __YODA_PHOTDATA__
#define __YODA_PHOTDATA__

#include <cmath>
#include <iostream>

class PhotData
{
   public:
      PhotData()
	 : xc_(0), yc_(0), a_(0), b_(0), pa_(0), ir1_(0), fixed_(0),
	   f_(0), df_(0), m_(0), dm_(0), sky_(0), dsky_(0), 
           ff_(0), dff_(0), fm_(0), dfm_(0), fsky_(0), dfsky_(0),
           alpha_(0), beta_(0), lum_(0)
      { }

      PhotData( const float xc, const float yc, 
		const float a, const float b, const float pa, const float ir1,
                const float fixed )
	 : xc_(xc), yc_(yc), a_(a), b_(b), pa_(pa), ir1_(ir1), fixed_(fixed),
	   f_(0), df_(0), m_(0), dm_(0), sky_(0), dsky_(0),
           ff_(0), dff_(0), fm_(0), dfm_(0), fsky_(0), dfsky_(0),
           alpha_(0), beta_(0), lum_(0)
      { }

      
      ~PhotData();

      float pa_rad() const
      { return pa_*M_PI/180.; }

      float pa_deg() const
      { return pa_; }
      
      void writecat( ::std::ostream& os );
      
      // position-angle is stored in DEG
      float xc_, yc_, a_, b_, pa_, ir1_, 
         fixed_;                            // coordinates and aperture size

      float f_, df_, m_, dm_, sky_, dsky_;  // flux, mag, sky and their errors
      float ff_, dff_, fm_, dfm_, 
         fsky_, dfsky_;                     // -"- from fixed circular aperture
      
      float alpha_, beta_, lum_;            // classifier output
};



#endif
